using System.ComponentModel.DataAnnotations;
using System.Text.Json.Serialization;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Mvc;
using Todo.Core.Application.Commands.Todo;
using Todo.Core.Application.Queries.Todos;
using Todo.Core.Interfaces.Persistence;

namespace Todo.Api.Controllers;

[Route("/api/todos")]
[Authorize]
public class TodosController : ApiController
{
    private readonly ITodoRepository _todoRepository;

    public TodosController(ITodoRepository todoRepository) => _todoRepository = todoRepository;

    public record CreateTodoRequest
    {
        [Required]
        [JsonPropertyName("title")]
        public string Title { get; init; }

        [JsonPropertyName("description")]
        public string? Description { get; init; }

        [JsonPropertyName("project")]
        public string? Project { get; init; }

        [JsonPropertyName("priority")]
        public string? Priority { get; init; }

        internal CreateTodoCommand To() => new(
            Title,
            Project,
            Description,
            Priority);
    }

    [HttpPost]
    public async Task<ActionResult<string>> CreateTodo(
        [FromBody] CreateTodoRequest request)
        => Ok(await Mediator.Send(request.To()));

    [HttpGet("{todoId}")]
    public async Task<ActionResult<TodoViewModel>> GetTodoById([FromRoute] string todoId)
        => await Mediator.Send(new GetTodoByIdQuery(todoId));

    [HttpGet]
    public async Task<ActionResult<IEnumerable<TodoViewModel>>> GetTodos([FromQuery] bool onlyActive = false)
    {
        if (onlyActive)
            return Ok(await Mediator.Send(new GetActiveTodosQuery()));

        return Ok(await Mediator.Send(new GetTodosQuery()));
    }

    [HttpPut("{todoId}")]
    public async Task<ActionResult> ReplaceTodo([FromRoute] string todoId, [FromBody] ReplaceTodoRequest request)
    {
        await Mediator.Send(request.To(todoId));
        return NoContent();
    }

    public record ReplaceTodoRequest
    {
        [Required]
        [JsonPropertyName("title")]
        public string Title { get; init; }

        [JsonPropertyName("description")]
        public string? Description { get; init; }

        [JsonPropertyName("project")]
        public string? Project { get; init; }

        [JsonPropertyName("status")]
        public bool Status { get; init; }

        [JsonPropertyName("created")]
        public long Created { get; init; } = 0;

        [JsonPropertyName("priority")]
        public string? Priority { get; init; }

        internal ReplaceTodoCommand To(string id) => new(
            id,
            Title,
            Project,
            Description,
            Status,
            Created,
            Priority);
    }
}